@@ -15,8 +15,8 @@ use crate::widget::style::Color as WidgetColor;
 use crate::widget::table::{StatefulTable, TableSize, TableState};
 use anyhow::{anyhow, Error as AnyhowError, Result};
 use colorsys::Rgb;
+use copypasta_ext::display::DisplayServer as ClipboardDisplayServer;
 use copypasta_ext::prelude::ClipboardProvider;
-use copypasta_ext::x11_fork::ClipboardContext;
 use std::collections::HashMap;
 use std::path::Path;
 use std::process::Command as OsCommand;
@@ -58,7 +58,7 @@ pub struct App<'a> {
 	/// Bottom margin value of the keys table.
 	pub keys_table_margin: u16,
 	/// Clipboard context.
-	pub clipboard: Option<ClipboardContext>,
+	pub clipboard: Option<Box<dyn ClipboardProvider>>,
 	/// GPGME context.
 	pub gpgme: &'a mut GpgContext,
 }
@@ -95,12 +95,12 @@ impl<'a> App<'a> {
 			keys_table_states: HashMap::new(),
 			keys_table_detail: KeyDetail::Minimum,
 			keys_table_margin: 1,
-			clipboard: match ClipboardContext::new() {
-				Ok(clipboard) => Some(clipboard),
-				Err(e) => {
-					eprintln!("failed to initialize clipboard: {:?}", e);
+			clipboard: match ClipboardDisplayServer::select().try_context() {
+				None => {
+					eprintln!("failed to initialize clipboard, no suitable clipboard provider found");
 					None
 				}
+				clipboard => clipboard,
 			},
 			gpgme,
 		})
@@ -315,17 +315,23 @@ impl<'a> App<'a> {
 			}
 			Command::ImportKeys(_, false) | Command::ImportClipboard => {
 				let mut keys = Vec::new();
+				let mut import_error = String::from("no files given");
 				if let Command::ImportKeys(ref key_files, _) = command {
 					keys = key_files.clone();
 				} else if let Some(clipboard) = self.clipboard.as_mut() {
-					keys = vec![clipboard
-						.get_contents()
-						.expect("failed to get clipboard contents")];
+					match clipboard.get_contents() {
+						Ok(content) => {
+							keys = vec![content];
+						}
+						Err(e) => {
+							import_error = e.to_string();
+						}
+					}
 				}
 				if keys.is_empty() {
 					self.prompt.set_output((
 						OutputType::Failure,
-						String::from("no files given"),
+						format!("import error: {}", import_error),
 					))
 				} else {
 					match self
@@ -840,13 +846,21 @@ impl<'a> App<'a> {
 							self.run_command(Command::Quit)?;
 						} else if let Some(clipboard) = self.clipboard.as_mut()
 						{
-							clipboard
-								.set_contents(content)
-								.expect("failed to set clipboard contents");
-							self.prompt.set_output((
-								OutputType::Success,
-								format!("{} copied to clipboard", copy_type),
-							));
+							self.prompt.set_output(
+								match clipboard.set_contents(content) {
+									Ok(_) => (
+										OutputType::Success,
+										format!(
+											"{} copied to clipboard",
+											copy_type
+										),
+									),
+									Err(e) => (
+										OutputType::Failure,
+										format!("clipboard error: {}", e),
+									),
+								},
+							);
 						} else {
 							self.prompt.set_output((
 								OutputType::Failure,
@@ -865,13 +879,18 @@ impl<'a> App<'a> {
 			}
 			Command::Paste => {
 				if let Some(clipboard) = self.clipboard.as_mut() {
-					self.prompt.clear();
-					self.prompt.text = format!(
-						":{}",
-						clipboard
-							.get_contents()
-							.expect("failed to get clipboard contents")
-					);
+					match clipboard.get_contents() {
+						Ok(content) => {
+							self.prompt.clear();
+							self.prompt.text = format!(":{}", content);
+						}
+						Err(e) => {
+							self.prompt.set_output((
+								OutputType::Failure,
+								format!("clipboard error: {}", e),
+							));
+						}
+					}
 				} else {
 					self.prompt.set_output((
 						OutputType::Failure,
